/* Encyklopedie jazyka C
 *
 * Kapitola Textové řetězce
 *
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#define VERZE "2.4"
#define NAZEV "Textové řetězce"

// TODO Prostuduj si obsah tohoto hlavičkového souboru! Jsou tam zaklínadla,
// bez kterých některé věci ve Windows nefungují správně!
// - barevný výstup
// - printg - český tisk (překódovává z UTF8 na CP852)
// - printh - český tisk + vyprávěcí mód
// - formátovací a formátové značky pro printf/scanf podle normy
// - obsluha nabídek v kapitolách
#include "pomocny-modul.h"

// Implementace nabídkového menu
#include "menu.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <ctype.h>
// #include <math.h>
// #include <float.h>
// #include <limits.h>

#include <locale.h>
#include <wchar.h>

// Hlavičky funkcí realizující jednotlivé kapitoly.

void textovyRetezec(void);
void stringJakoPole(void);
void stringyAUkazatele(void);
void stringyAPodprogramy(void);

void formatovanyTisk(void);
void formatovaneCteni(void);
void tokenizaceStringu(void);

void knihovnaString(void);
void ceskaLokalizace(void);
void sirokeStringy(void);

int main(void)
{
  povolBarevnyVystup();
//  prepniRychlostVypraveni(2);

  Tmenu * menu = menuVytvor(
    NADPIS(
      "==============================================================\n"
      "Encyklopedie jazyka C - " NAZEV " (verze " VERZE ")\n"
      "(c) David Martinek, 2023\n"
      "Zároveň s programem sleduj i zdrojový text!\n"
      "==============================================================\n"
    )
    , zobrazMod
  );

  if (menu == NULL)
  {
    printg("Není dost paměti ani pro výrobu menu. Končím program.");
    return 0;
  }

  menuPridej(menu,   0, "\n" SEKCE("Jazykové konstrukce") , NULL);
  menuPridej(menu, 'A', "Textový řetězec - string\n", textovyRetezec);
  menuPridej(menu, 'B', "String jako pole\n", stringJakoPole);
  menuPridej(menu, 'C', "String a ukazatel, struktura, pole\n", stringyAUkazatele);
  menuPridej(menu, 'D', "Stringy a podprogramy\n", stringyAPodprogramy);

  menuPridej(menu,   0, "\n" SEKCE("Tisk a čtení") , NULL);
  menuPridej(menu, 'E', "Formátovaný tisk\n", formatovanyTisk);
  menuPridej(menu, 'F', "Formátované čtení\n", formatovaneCteni);
  menuPridej(menu, 'G', "Tokenizace stringů\n", tokenizaceStringu);

  menuPridej(menu,   0, "\n" SEKCE("Knihovní funkce") , NULL);
  menuPridej(menu, 'H', "Knihovna string.h\n", knihovnaString);
  menuPridej(menu, 'I', "Česká lokalizace (pro pokročilé)\n", ceskaLokalizace);
  menuPridej(menu, 'J', "Široké znaky a stringy (pro odvážné)\n", sirokeStringy);
//  menuPridej(menu, 'K', "\n", test);
//  menuPridej(menu, 'L', "\n", test);

  menuPridej(menu,   0, "\n" SEKCE("Nastavení programu"), NULL);
  menuPridej(menu, '0', "Nastav vyprávěcí mód\n", nastavRychlostVypraveni);

  while (menuVolba(menu) != 'Q') {}

  printg("Končím program.\n");
  menuZrus(menu);
  return 0;
}


////////////////////////////////////////////////////////////////////////////////
void textovyRetezec(void)
{
  zacatekKapitoly("Textový řetězec - string");

  printh(
    "- Český termín \"textový řetězec\" se někdy zkracuje jako \"řetězec\",\n"
    "  ale často se přejímá anglický termín \"string\".\n"
    "- Textový řetězec, neboli string je v C libovolně dlouhá posloupnost\n"
    "  znaků ukončená nulovým znakem " KK("'\\0'") ".\n"
    "- Ukládá se do pole znaků.\n"
    "\n"
  );

  if (navratNeboSekce("Řetězcové literály")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "\"\"\n"
      )
      "- Prázdný string zabírá 1 bajt, protože je v něm uložen znak " KK("'\\0'") " (ASCII kód 0).\n"
      "- Nulový znak se do literálů nepíše. Překladač jej doplní na konec automaticky.\n"
      "\n"
      KOD(
      "\"A\"\n"
      )
      "- String délky 1 znak zabírá 2 bajty.\n"
      "- Pozor! Neplést se znakovou konstantou " KK("'A'") ". Ta zabírá 1 bajt a používá\n"
      "  jednoduché uvozovky místo dvojitých.\n"
      "\n"
      KOD(
      "\"ahoj\"\n"
      )
      "- String délky 4 znaky zabírá 5 bajtů.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Dělené literály")) return;
  if (pokracovat())
  {
    char * text1 = "Ahoj,"             " jak se mas?\n";
    char * text2 = "Skakal pes,\n"
                   "pres oves.\n";

    printh(
      "\n"
      KOD(
      "\"Ahoj,\"  \" jak se mas?\\n\"\n"
      )
      "  => %s\n"
      KOD(
      "\"Skakal pes,\\n\"\n"
      "\"pres oves.\\n\"\n"
      )
      "  => %s\n"
      , text1
      , text2
    );
    printh(
      "- Dva literály oddělené pouze bílými znaky tvoří jeden string.\n"
      "- Tip: Víceřádkové literály je přehlednější zapisovat skutečně\n"
      "  na více řádcích. Nezapomínej ale použít znaky " KK("\\n") ".\n "
      "\n"
      "- V jazyce C musejí být počáteční i koncová uvozovka na stejném řádku.\n"
      // Ještě existuje možnost použít na konci řádku opačné lomítko (\).
      // Následující řádek se pak považuje za pokračování předchozího.
      // Není to ale praktické, protože u stringů je pořád potřeba ručně
      // vkládat symbol \n. Navíc pak nejde rozumně odsazovat.
      // Tip: Já pro víceřádkové stringy využívám možností textového editoru.
      // V Code::Blocks můžu pomocí Ctrl+D snadno vyrobit duplikát řádku.
      // Levý Alt+šipka nahoru/dolů - posouvá aktuální řádek nahoru/dolů
      // Dále jde zapnout funkci blokového výběru a kurzoru
      // Settings - Editor... - General settings - Selections -
      //   - Enable virtual space for rectangle selections
      //   - Allow multiple selections
      //     - Enable typing (and deleting) in multiple selections simultaneously
      // Levý Alt+Shift+šipky - blokový víceřádkový výběr - funguje i se schránkou
      // a jde takto vkládat i do více řádků zároveň.
      // Rovněž takto jde psát a mazat zároveň ve více řádcích.
      "\n"
    );
  }

  if (navratNeboSekce("Speciální znaky")) return;
  if (pokracovat())
  {
    printh(
      "- Speciální znaky mohou být ve stringu kdekoli.\n"
      "- Speciální znaky se zapisují v escapované podobě, tj. s opačným lomítkem.\n"
      "  Například:\n"
      "  " KK("\\n") " - konec řádku, odřádkování\n"
      "  " KK("\\t") " - horizontální tabulátor\n"
      "  " KK("\\v") " - vertikální tabulátor (nefunguje ve Windows)\n"
      "  " KK("\\\\") " - opačné lomítko\n"
      "  " KK("\\\"") " - uvozovka\n"
      "  " KK("\\0") " - nulový znak - není potřeba jej explicitně zapisovat\n"
      "- Pozor! Nulový znak " KK("'\\0'") " má ASCII kód 0, kdežto " KK("'0'") " má kód %d.\n"
      "\n"
      , '0'
    );
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void stringJakoPole(void)
{
  zacatekKapitoly("Stringy a pole");

  printh(
      "- Pro ukládání stringů slouží pole znaků.\n"
      "- Pole znaků ale není automaticky stringem.\n"
      "- Pojem string je spíš popisem významu než datového typu.\n"
      "- String se z pole znaků stane až poté, co jej tak začneme používat,\n"
      "  tedy uložíme do něj posloupnost znaků končící znakem " KK("'\\0'") ".\n"
    "\n"
  );

  if (navratNeboSekce("Deklarace a definice")) return;
  if (pokracovat())
  {
//    char text[10+1];

    printh(
      "\n"
      KOD(
      "  char text[10+1];\n"
      )
      "\n"
      "- Proměnná " KK("text") " může (ale nemusí) sloužit pro uložení stringu.\n"
      "- String uložený do proměnné " KK("text") " může být dlouhý 0 až 10 znaků.\n"
      "\n"
      "- Konvence: Vyhrazením velikosti 10+1 dávám najevo, že sem plánuji\n"
      "  ukládat stringy maximálně 10 znaků dlouhé a +1 je prostor pro uložení\n"
      "  ukončovacího znaku " KK("'\\0'") ".\n"
      "\n"
    );
  }

  if (navratNeboSekce("Inicializace funkcemi strcpy, strncpy")) return;
  if (pokracovat())
  {
    char text[20+1];
    //text = "Toto je text."; // Syntaktická chyba
    strcpy(text, "Toto je text.");
    strncpy(text, "Toto je text.", 20+1);
    printh(
      "\n"
      KOD(
      "  char text[20+1];\n"
      "  //text = \"Toto je text.\"; // Syntaktická chyba\n"
      "  strcpy(text, \"Toto je text.\");\n"
      "  strncpy(text, \"Toto je text.\", 20+1);\n"
      )
      "\n"
      "  => text: %s\n"
      "\n"
      "- Proměnnou typu pole nejde změnit přiřazením.\n"
      "- Kopie stringů se dělá pomocí funkce " KK("strcpy") ".\n"
      "  Pozor! V cílovém poli musí být dostatek místa.\n"
      "- Kopie stringů pomocí " KK("strncpy") " je bezpečnější, protože lze předepsat,\n"
      "  kolik bajtů maximálně se bude kopírovat.\n"
      "\n"
      "- Při práci se stringovými literály je lepší použít inicializátor.\n"
      "\n"
      , text
    );
  }

  if (navratNeboSekce("Inicializátor")) return;
  if (pokracovat())
  {
    char text1[20+1] = "Te pero, sobrero!";
    char text2[] = "Dal by sis toust?";
    char text3[] = {'S', 'm', 'e', 'g', 'h', 'e', 'a', 'd', '!', '\0'};

    printh(
      "\n"
      KOD(
      "char text1[20+1] = \"Te pero, sobrero!\";\n"
      "char text2[] = \"Dal by sis toust?\";\n"
      "char text3[] = {'S', 'm', 'e', 'g', 'h', 'e', 'a', 'd', '!', '\\0'};\n"
      )
      "\n"
      "=> text1: %s\n"
      "=> text2: %s\n"
      "=> text3: %s\n"
      "\n"
      "- Inicializátor jde použít výhradně při definici proměnné.\n"
      "- Tradiční inicializátor pole " KK("{}") " sice jde použít, ale u stringů\n"
      "  je velmi nepraktický. Aby vznikl string, je potřeba vložit znak " KK("'\\0'") ".\n"
      "- Inicializaci lze provést pomocí stringového literálu.\n"
      "- Při inicializaci dojde ke kopii celého stringu včetně koncové nuly.\n"
      "- Pokud je vynechán rozměr pole, jeho délka se dopočítá automaticky tak,\n"
      "  aby se do pole vešel celý string i s koncovým nulovým znakem.\n"
      "\n"
      "- Poznámka: Pozor na stringy s českou diakritikou! Jazyk C používá\n"
      "  jednobajtové znaky a čeština (zvláště ve Windows) může způsobovat\n"
      "  problémy. Viz část o lokalizaci.\n"
      "\n"
      , text1
      , text2
      , text3
    );
  }

  if (navratNeboSekce("Výhody použití stringů v polích")) return;
  if (pokracovat())
  {
    printh(
      "- Se stringem jde zacházet jako s polem.\n"
      "- Jednotlivé znaky jde indexovat.\n"
      "- Znaky v poli jde modifikovat. (U konstantních proměnných to nejde.)\n"
      "\n"
      KOD(
      "  char abeceda[] = \"abcdefghijklmnopqrstuvwxyz\";\n"
      "  printf(\"  => %%s\\n\", abeceda);\n"
      "  for (int i = 0; abeceda[i] != '\\0'; ++i)\n"
      "  {\n"
      "    abeceda[i] = toupper(abeceda[i]);\n"
      "  }\n"
      "  printf(\"  => %%s\\n\", abeceda);\n"
      )
      "\n"
    );
    char abeceda[] = "abcdefghijklmnopqrstuvwxyz";
    printf("  => %s\n", abeceda);
    for (int i = 0; abeceda[i] != '\0'; ++i)
    {
      abeceda[i] = toupper(abeceda[i]);
    }
    printf("  => %s\n\n", abeceda);
  }

  if (navratNeboSekce("Nevýhody použití stringů v polích")) return;
  if (pokracovat())
  {
    printh(
      "- Modifikovatelné stringy způsobují synchronizační problémy\n"
      "  v paralelních programech.\n"
      "=> Moderní jazyky s podporou paralelismu mívají pro stringy speciální\n"
      "   typ, jehož hodnoty nejdou modifikovat.\n"
      // Říká se tomu immutable type - neměnný typ. Pokus o modifikaci stringu
      // vede k vytvoření nového, změněného stringu. Další důvod, pro použití
      // immutable stringů je jejich snazší použití jako klíčů v hašovacích
      // tabulkách (které jazyk C sám o sobě neřeší). Jazyky jako Java a Python
      // si vnitřně tvoří zásobárnu (pool) použitých stringů a nikdy nevytvoří
      // dva totožné stringy - pokus o vytvoření stejného stringu vede k vrácení
      // odkazu na původní, již hotový string. Využívají se tu návrhové vzory
      // Singleton a Pool.
      // Na jednu stranu řeší immutable stringy hodně problémů v paralelních
      // programech, ale cenou za tohle chování je velké plýtvání pamětí
      // a implementační náročnost.
      // Jazyk C není primárně určen pro paralelní programování, takže používá
      // modifikovatelné stringy. Pokud programátor má potřebu programovat v C
      // paralelně (například pomocí OpenMP), musí si tohle ohlídat sám.
    );
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////
void stringyAUkazatele(void)
{
  zacatekKapitoly("Stringy a ukazatel, struktura, pole");

  printh(
    "- Díky příbuznosti ukazatelů a polí jde se stringem pracovat přes ukazatel.\n"
    "- Pro uložení stringů je možné používat dynamicky alokovaná pole.\n"
    "- Má to však důsledky, kvůli kterým je třeba s takovými stringy pracovat\n"
    "  s větší pozorností.\n"
    "\n"
  );

  if (navratNeboSekce("Pozor na co to odkazuje")) return;
  if (pokracovat())
  {
    char text[] = "toto je pole"; // inicializátor zkopíruje konstantu do pole
    char *text1 = text;                // odkaz na pole text
    char *text2 = "toto je konstanta"; // odkaz na konstantu

    text1[0] = 'T'; // Toto se provede.
    //text2[0] = 'T'; // Toto havaruje. Zkus to odkomentovat a spustit.

    printh(
      "\n"
      KOD(
      "  char text[] = \"toto je pole\"; // inicializátor zkopíruje konstantu do pole\n"
      "  char *text1 = text;                // odkaz na pole text\n"
      "  char *text2 = \"toto je konstanta\"; // odkaz na konstantu\n"
      "\n"
      "  text1[0] = 'T'; // Toto se provede.\n"
      "  //text2[0] = 'T'; // Toto havaruje.\n"
      )
      "\n"
      "  =>  text: %s\n"
      "  => text1: %s\n"
      "  => text2: %s\n"
      "\n"
      "- Práce s ukazatelem na string se liší podle toho, na co přesně ukazatel\n"
      "  odkazuje.\n"
      "- Proměnná " KK("text1") " odkazuje na běžné pole, proto přes ní jde provádět\n"
      "  změny. Ty se projeví v původním poli " KK("text") ".\n"
      "- Proměnná " KK("text2") " odkazuje na konstantu, proto v takovém stringu\n"
      "  nejde provádět žádné změny. Skončily by havárií.\n"
      "\n"
      "- Na tento problém je potřeba pamatovat zejména u položek struktur\n"
      "  a u parametrů funkcí.\n"
      "\n"
      , text
      , text1
      , text2
    );
  }

  if (navratNeboSekce("String jako položka struktury")) return;
  if (pokracovat())
  {
//    typedef struct osoba
//    {
//      char jmeno[20+1];
//      char * prijmeni;
//      const char * adresa;
//    } Tosoba;

    printh(
      "\n"
      KOD(
      "typedef struct osoba\n"
      "{\n"
      "  char jmeno[20+1];\n"
      "  char * prijmeni;\n"
      "  const char * adresa;\n"
      "} Tosoba;\n"
      )
      "\n"
      "- Položka " KK("jmeno") " v ukázce je pole.\n"
      "  - Má omezenou délku.\n"
      "  - Při kopírování struktury se udělá i kopie celého pole.\n"
      "  - Je editovatelná.\n"
      "\n"
      "- Položka " KK("prijmeni") " je ukazatel.\n"
      "  - Je potřeba mít jistotu, na co bude ukazatel odkazovat.\n"
      "  - Pokud odkazuje na dynamicky alokované pole, je třeba se postarat\n"
      "    o uvolňování paměti.\n"
      "  - Pokud bude odkazovat na konstantní literál, nesmí dojít k editaci!\n"
      "  - V celém programu to musí být konzistentní, jinak bude docházet\n"
      "    k chybám a haváriím.\n"
      "  - Při kopírování struktury se udělá kopie ukazatele, ne stringu!\n"
      "\n"
      "- Položka " KK("adresa") " je ukazatel na konstantní pole.\n"
      "  - Tohle je závazek, že string nikdy nebude editován.\n"
      "  - Při kopii struktury se dělá kopie ukazatele, ne stringu.\n"
      "    Zde to obvykle nevadí. Pokud ale originál zanikne, vede to\n"
      "    k chybám a haváriím.\n"
      /* Tuto variantu jsem použil v tomto programu při implementaci menu.
      Předpokládám totiž, že se do položek menu budou dávat jen odkazy na
      konstantní literály. Pokud by se položky načítaly třeba ze souboru,
      musel bych zařídit alokaci a dealokaci polí pro stringy nějak bokem.
      */
      "\n"
    );
  }

  if (navratNeboSekce("Pole stringů")) return;
  if (pokracovat())
  {
//    char pole1[10][20+1];
//    char * pole2[10];
//    const char * pole3[10];

    printh(
      "\n"
      KOD(
      "  char pole1[10][20+1];\n"
      "  char * pole2[10];\n"
      "  const char * pole3[10];\n"
      )
      "\n"
      "- Pro pole stringů platí přesně totéž, co pro položky struktur.\n"
      "- Proměnná " KK("pole1") " vyhradí fyzický prostor pro 10 stringů délky\n"
      "  maximálně 20 znaků.\n"
      "- Proměnná " KK("pole2") " je pole ukazatelů na stringy vyrobené jinde.\n"
      "  Je potřeba vědět, jesli jsou modifikovatelné nebo ne.\n"
      "- Proměnná " KK("pole3") " je pole ukazatelů na nemodifikovatelné stringy,\n"
      "  například konstantní literály.\n"
      "\n"
      "- Pozor na záměnu pole znaků a pole ukazatelů! Častý omyl!\n"
      "- Kombinovat " KK("*") " a " KK("[]") " takzvaně \"pro jistotu\" fakt není dobrý nápad.\n"
      "\n"
      KOD(
      "  char str[];    // pole znaků - může v něm být string\n"
      "  char * arr[];  // pole ukazatelů - možná ukazatelů na stringy\n"
      )
      "\n"
    );
  }


  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////

/** Změní všechna malá písmena v zadaném textu na velká písmena.
 *
 * \param text char* String, který se bude modifikovat.
 */
void prevedNaVelke(char *text)
//void prevedNaVelke(char text[])
{
  for (int i = 0; text[i] != '\0'; ++i)
  {
    text[i] = toupper(text[i]);
  }
}

/** Změní všechna malá písmena na velká a vrátí ukazatel na začátek stringu.
 *
 * Díky tomu, že funkce vrátí ukazatel na začátek modifikovaného stringu,
 * jde tato funkce použít ve výrazech - například jako parametr funkce printf.
 *
 * Pozor! Pokud se ve výrazu nebo v printf použije vícekrát, musí se pokaždé
 * volat s jiným argumentem! (Proč asi?)
 *
 * \param text char* String, který se bude modifikovat.
 * \return char* Vrací ukazatel na začátek zadaného stringu.
 */
char * naVelke(char *text)
{
  for (int i = 0; text[i] != '\0'; ++i)
  {
    text[i] = toupper(text[i]);
  }
  return text;
}

/** Vrací počet mezer v zadaném stringu.
 *
 * \param text const char* Zadaný string - const je závazek, že nebude měněn.
 * \return int Vrací počet mezer v zadaném stringu.
 *
 */
int pocetMezer(const char *text)
{
  int pocet = 0;
  for (int i = 0; text[i] != '\0'; ++i)
  {
    if (text[i] == ' ')
      pocet += 1;
  }
  return pocet;
}

void stringyAPodprogramy(void)
{
  zacatekKapitoly("Stringy a podprogramy");

  printh(
    "- Platí téměř totéž, co pro práci s jednorozměrnými poli a podprogramy.\n"
    "\n"
    "- Protože string je ukončen nulovým znakem, často není potřeba do funkcí\n"
    "  předávat spolu se stringem jeho délku.\n"
    "- Je ale vhodné rozlišovat, zda funkce bude parametr typu string\n"
    "  modifikovat nebo ne.\n"
    "\n"
    "- Funkce nemůže vracet pole, ale může vracet ukazatel.\n"
    "- Pokud vrací ukazatel, nesmí to být adresa lokální proměnné.\n"
    "\n"
  );

  if (navratNeboSekce("Parametr typu modifikovatelný string")) return;
  if (pokracovat())
  {
    char text1[] = "toto je modifikovatelne pole";
//    char *text2 = "toto je konstanta";
    prevedNaVelke(text1);
//    prevedNaVelke(text2);  // Toto havaruje. Odkomentuj to a vyzkoušej to.
//    prevedNaVelke("toto je konstanta"); // Toto havaruje.

    printh(
      "\n"
      KOD(
      "void prevedNaVelke(char *text);\n"
      "void prevedNaVelke(char text[]);\n"
      )
      "\n"
      "- Oba prototypy jsou prakticky totožné.\n"
      "- První varianta je běžnější.\n"
      "\n"
      "- Při volání se jako argument musí předat modifikovatelný string.\n"
      "- Pozor! Předá-li se konstantní literál, nezpůsobí to ani varování\n"
      "  překladače. Způsobí to ale havárii programu!\n"
      "\n"
      KOD(
      "  char text1[] = \"toto je modifikovatelne pole\";\n"
      "  char *text2 = \"toto je konstanta\";\n"
      "  prevedNaVelke(text1);\n"
      "  // prevedNaVelke(text2);  // Toto havaruje.\n"
      "  // prevedNaVelke(\"toto je konstanta\"); // Toto havaruje.\n"
      )
      "\n"
      "  => %s\n"
      "\n"
      , text1
    );
  }

  if (navratNeboSekce("Finta s funkcí vracející zadaný argument")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "char * naVelke(char *text)\n"
      "{\n"
      "  for (int i = 0; text[i] != '\\0'; ++i)\n"
      "  {\n"
      "    text[i] = toupper(text[i]);\n"
      "  }\n"
      "  return text;\n"
      "}\n"
      )
      "\n"
      "- Funkce může vracet ukazatel.\n"
      "- V tomto případě vrací ukazatel na string, který byl zadán při volání.\n"
      "- Cílem je umožnit použití výsledku ve výrazech, respektive při tisku\n"
      "  funkcí printf.\n"
      // Pozor! Použije-li se taková funkce v jednom výrazu nebo při tisku
      // vícekrát, musí se použít s různými argumenty. U této funkce by to
      // byl jen zbytečný výpočet, ale u jiných funkcí s touto fintou to může
      // vést k nechtěnému chování. Viz použití funkce objektNaString
      // v kapitole Struktury.
      "\n"
      KOD(
      "  char text[] = \"modifikovatelny text puvodne malymi pismeny\";\n"
      "  printf(\"  => %%s\\n\", naVelke(text));\n"
    )
      "\n"
    );

    char text[] = "modifikovatelny text puvodne malymi pismeny";
    printf("  => %s\n\n", naVelke(text));
  }

  if (navratNeboSekce("Parametr typu string pro čtení")) return;
  if (pokracovat())
  {
    int pocet = pocetMezer("Skakal pes, pres oves, pres zelenou louku.");

    printh(
      "\n"
      KOD(
      "int pocetMezer(const char *text)\n"
      "{\n"
      "  int pocet = 0;\n"
      "  for (int i = 0; text[i] != '\\0'; ++i)\n"
      "  {\n"
      "    if (text[i] == ' ')\n"
      "      pocet += 1;\n"
      "  }\n"
      "  return pocet;\n"
      "}\n"
      )
      "\n"
      "- V této funkci je parametr typu ukazatel na konstantní znak,\n"
      "  respektive pole konstantních znaků.\n"
      "- Klíčové slovo " KK("const") " vyjadřuje závazek, že funkce nebude měnit\n"
      "  žádný znak stringu.\n"
      "  => Parametr " KK("text") " je pouze pro čtení.\n"
      "  => Jako argument jde bezpečně použít i konstantní literál.\n"
      "\n"
      KOD(
      "  int pocet = pocetMezer(\"Skakal pes, pres oves, pres zelenou louku.\");\n"
      )
      "\n"
      "  => pocet: %d\n"
      "\n"
      , pocet
    );
  }


  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void formatovanyTisk(void)
{
  zacatekKapitoly("Formátovaný tisk");

  printh(
    "- Pro formátovaný tisk se používá rodina funkcí " KK("printf") ", " KK("fprintf") ", " KK("sprintf") ", ...\n"
    "- Formátovací značkou pro stringy je " KK("%%s") ".\n"
    "\n"
  );

  if (navratNeboSekce("Formátovací značka %%s")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  %%<zarovnání><.oříznutí>s\n"
      )
      "\n"
      KK("%%s") " - Základní formátovací značka pro stringy\n"
      "\n"
      KK("<zarovnání>") "\n"
      "- Nepovinná délka zarovnání - nadbytečný prostor zarovná mezerami.\n"
      "- Normálně se zarovnává doprava.\n"
      "- Záporná hodnota zarovnání zarovnává doleva.\n"
      "- Pokud je zarovnání menší než délka stringu, ignoruje se.\n"
      "\n"
      KK("<.oříznutí>") "\n"
      "- Nepovinná šířka oříznutí.\n"
      "- Příliš dlouhý text jde oříznout uvedením celého čísla za tečkou.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázky formátovaného tisku stringů")) return;
  if (pokracovat())
  {
    char text[] = "Popocatepetl";
    printh(
      "\n"
      "  " KK("|%%s|     ") " => |%s|\n"
      "  " KK("|%%20s|   ") " => |%20s|\n"
      "  " KK("|%%-20s|  ") " => |%-20s|\n"
      "  " KK("|%%5s|    ") " => |%5s|\n"
      "  " KK("|%%.5s|   ") " => |%.5s|\n"
      "  " KK("|%%10.5s| ") " => |%10.5s|\n"
      "  " KK("|%%-10.5s|") " => |%-10.5s|\n"
      "\n"
      , text
      , text
      , text
      , text
      , text
      , text
      , text
    );
  }

  if (navratNeboSekce("Zarovnání zadané proměnnou")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  %%*.*s\n"
      )
      "\n"
      "- Na pozici hvězdiček se dosadí hodnoty dalších argumentů udávajících\n"
      "  zarovnání a oříznutí.\n"
      "\n"
      KOD(
      "  const char * text = \"Popocatepetl\";\n"
      "  int zarovnani = 20;\n"
      "  int oriznuti = 5;\n"
      "  printf(\"  => |%%*s|\\n\", zarovnani, text);\n"
      "  printf(\"  => |%%*.*s|\\n\", zarovnani, oriznuti, text);\n"
      "  printf(\"  => |%%-*.*s|\\n\", zarovnani, oriznuti, text);\n"
      )
      "\n"
    );

    const char * text = "Popocatepetl";
    int zarovnani = 20;
    int oriznuti = 5;
    printf("  => |%*s|\n", zarovnani, text);
    printf("  => |%*.*s|\n", zarovnani, oriznuti, text);
    printf("  => |%-*.*s|\n", zarovnani, oriznuti, text);
    printf("\n");
  }

  if (navratNeboSekce("Na co si dávat pozor")) return;
  if (pokracovat())
  {
    printh(
      "- Nenahrazuj formátovací řetězec za proměnnou.\n"
      "\n"
      KOD(
      "  char text[] = \"  => Dan dela 30 %%darovane castky.\\n\";\n"
      "  printf(text); // Špatně! %%d bude očekávat další argument => může havarovat\n"
      "  printf(\"%%s\", text); // Tohle je správně. \n"
      )
      "\n"
    );

// Tohle je drobný podvod. Protože chci ukázat, jak se to nemá dělat, tak
// pro pár řádků vypnu hlídání této bezpečností chyby.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wformat-security"

    // Smysluplná značka může vzniknout třeba překlepem...
    char text[] = "  => Dan dela 30 %darovane castky.\n";
    printf(text); // Špatně! %d bude očekávat další argument => může havarovat
    printf("%s", text); // Tohle je správně.

#pragma GCC diagnostic pop

    printh(
      "\n"
      "- Pozor na speciální znaky a %% ve formátovacím řetězci.\n"
      "- Speciální znaky se zapisují v escapované podobě s opačným lomítkem,\n"
      "  např. \\\", \\n, \\t.\n"
      "- Znak %% je nutné mít ve formátovacím řetězci zdvojený %%%%, jinak se bude\n"
      "  považovat za formátovací značku.\n"
      "- Znak opačného lomítka \\ se musí tisknou zdvojený \\\\, jinak se bude\n"
      "  následující znak považovat za speciální znak.\n"
      "\n"
      KOD(
      "  printf(\"  => Specialni znaky: \\\", \\\\, %%%%, \\n x \\\\n, \\t x \\\\t\\n\");\n"
      )
      "\n"
    );
    printf("  => Specialni znaky: \", \\, %%, \n x \\n, \t x \\t\n");

    printh(
      "\n"
      "- V řetězci, který se tiskne značkou %%s, už nebudou zdvojené %%,\n"
      "  escapované speciální znaky ale fungují stejně.\n"
      "\n"
      KOD(
      "char spec[] = \"Specialni znaky: \\\", \\\\, %, \\n x \\\\n, \\t x \\\\t\";\n"
      "printf(\"  => %%s\\n\", spec);\n"
      )
      "\n"
    );
    char spec[] = "Specialni znaky: \", \\, %, \n x \\n, \t x \\t";
    printf("  => %s\n\n", spec);
  }

  if (navratNeboSekce("Tisk do stringu")) return;
  if (pokracovat())
  {
    printh(
      "- Pomocí funkcí " KK("sprintf") ", " KK("snprintf") " lze formátovaně tisknout do pole znaků.\n"
      "- V cílovém poli musí být dost místa pro uložení výsledného stringu.\n"
      "- Funkce " KK("snprinf") " je bezpečnější, protože umožňuje zadat délku cílového\n"
      "  pole (včetně místa pro koncový nulový znak).\n"
      "- Využití spočívá v algoritmické výrobě\n"
      "  - formátových řetězců pro funkci " KK("scanf") ",\n"
      // Standard ISO C99 nepodporuje značku %ms, která dělá vnitřně alokaci pomocí malloc.
      // Tato značka je v GNU C rozšíření.
      // Z toho důvodu dává smysl generovat formátový string s různými délkami
      // načítaných stringů algoritmicky pomocí snprintf.
      "  - formátovacích řetězců pro funkci " KK("printf") ",\n"
      // Pro printf to není tolik potřeba, protože jde formátovat pomocí proměnných.
      "  - formátovaných textů pro pozdější použití.\n"
      "\n"
      KOD(
      "  char text[100+1];\n"
      "  snprintf(text, 100+1, \"jmeno: %%s, vaha: %%g kg\", \"Azor\", 56.4);\n"
      "  printf(\"  => %%s\\n\", text);\n"
      )
      "\n"
    );

    char text[100+1];
    snprintf(text, 100+1, "jmeno: %s, vaha: %g kg", "Azor", 56.4);
    printf("  => %s\n", text);
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void formatovaneCteni(void)
{
  zacatekKapitoly("Formátované čtení");

  printh(
    "- Pro formátované čtení se používá rodina funkcí " KK("scanf") ", " KK("fscanf") ", " KK("sscanf") ", ...\n"
    "- Formátovou značkou pro stringy je " KK("%%Ns") ".\n"
    "\n"
    "- Funkce " KK("scanf") " má funkční hodnotu - vrací počet úspěšně zpracovaných\n"
    "  značek.\n"
    "- V případě úspěšného čtení musí funkce " KK("scanf") " vrátit stejnou hodnotu,\n"
    "  jako je počet značek ve formátovém stringu.\n"
    "- Pozor! Při chybném čtení se čtecí hlava zastaví na chybné hodnotě\n"
    "  a opakové pokusy o čtení ji nikam neposunou. Řeší se to značkou " KK("%%*s") ".\n"
    "  Viz dále.\n"
    "\n"
  );

  if (navratNeboSekce("Čtení slov - značka %%Ns")) return;
  if (pokracovat())
  {
    printh(
      "- Čte stringy po slovech - zastaví se na bílém znaku (mezera, tabulátor,\n"
      "  nový řádek apod.) nebo na konci vstupu.\n"
      "- Čtení stringů včetně mezer se dělá jinak. Viz dále.\n"
      "\n"
      KOD(
      "  %<maximální délka>s\n"
      )
      "\n"
      "  %%s\n"
      "  - V této podobě značku NIKDY nepoužívat!\n"
      "  - Čtení neomezené délky je nebezpečné. Je-li vstupem slovo delší\n"
      "    než cílové pole, dojde k zápisu mimo toto pole => možná havárie.\n"
      "  - Typická příčina známé bezpečnostní chyby \"buffer overflow\".\n"
      "\n"
      "  <maximální délka>\n"
      "  - Omezí počet znaků, které se budou číst.\n"
      "  - Pole pro zápis musí být alespoň o jedničku větší - pro koncovou nulu.\n"
      "  - Kratší slovo,\n"
      "    - se načte a v cílovém poli se ukončí nulovým znakem,\n"
      "    - poslední bajty cílového pole zůstanou nevyužity.\n"
      "  - Delší slovo,\n"
      "    - se nenačte celé - pouze zadaný počet znaků,\n"
      "    - za ně se do cílového pole zapíše ukončovací nula,\n"
      "    - na vstupu zůstane zbytek nepřečteného slova.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka - čtení slova")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  printf(\"  Zadej slovo: \");\n"
      "  char slovo[10+1];\n"
      "  int kod = scanf(\"%%10s\", slovo);\n"
      "  pozerKonecRadku(); // Kdyby po uživateli něco zbylo\n"
      "  printf(\"slovo: |%%-10s|, kod: %%d (kod=1 je ok, jinak chyba)\\n\"\n"
      "         , slovo, kod);\n"
      )
      "\n"
    );

    char znovu = 'z';
    while (znovu == 'z')
    {
      printf("  Zadej slovo: ");
      char slovo[10+1];
      int kod = scanf("%10s", slovo);

      pozerKonecRadku(); // Kdyby po uživateli něco zbylo
      printf(
        "\n"
        "  => slovo: |%-10s|, kod: %d (kod=1 je ok, jinak je chyba)\n"
        "\n"
        , slovo, kod
      );

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

    //char *str;
    //scanf("%ms", &str); //warning: ISO C does not support the 'm' scanf flag

  if (navratNeboSekce("Čtení s oddělovači - značka %%N[množina]")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  %%<maximální délka>[množina]\n"
      )
      "\n"
      "  [množina]\n"
      "  - Čte posloupnost znaků vyjmenovaných mezi hranatými závorkami.\n"
      "  - Jde použít intervaly, např. a-z, A-Z, 0-9.\n"
      "  - Čtení končí, když narazí na jakýkoli znak, který není vyjmenován.\n"
      "\n"
      "  [^množina]\n"
      "  - Čte posloupnost znaků, které nejsou vyjmenované v hranatých závorkách.\n"
      "  - Čtení končí, když narazí na jakýkoli znak, který je vyjmenován.\n"
      "  - Dá se použít pro vyjmenování oddělovačů.\n"
      "  - Když mezi oddělovači nebude mezera, bude se načítat včetně mezer.\n"
      "\n"
      "  " KK("%%10[a-z]        ") " => Čte max. 10 znaků malé anglické abecedy.\n"
      "  " KK("%%10[0-9a-fA-F]  ") " => Čte max. 10 hexa číslic.\n"
      "  " KK("%%10[^,;.]       ") " => Čte max. 10 znaků, dokud nenarazí na ',', ';' nebo '.'.\n"
      "  " KK("%%10[^ \\t\\v\\n\\r] ") " => Ekvivalent %%10s\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka - text a oddělovače")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  printf(\"  Zadej dva texty s mezerami oddelene carkou: \");\n"
      "  char str1[20+1];\n"
      "  char str2[20+1];\n"
      "  int kod = scanf(\"%%20[^,],%%20[^\\n]\", str1, str2);\n"
      "  pozerKonecRadku(); // Kdyby po uživateli něco zbylo\n"
      "  printf(\"str1: |%%s|, str2: |%%s|, kod: %%d (kod=2 je ok, jinak chyba)\\n\"\n"
      "    , str1, str2, kod);\n"
      )
      "\n"
    );

    char znovu = 'z';
    while (znovu == 'z')
    {
      printg("  Zadej dva texty s mezerami oddělené čárkou: ");
      char str1[20+1];
      char str2[20+1];
      int kod = scanf("%20[^,],%20[^\n]", str1, str2);
      pozerKonecRadku(); // Kdyby po uživateli něco zbylo
      printf(
        "\n"
        "  => str1: |%s|, str2: |%s|, kod: %d (kod=2 je ok, jinak chyba)\n"
        "\n"
        , str1, str2, kod
      );

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  if (navratNeboSekce("Mezery ve formátovém řetězci")) return;
  if (pokracovat())
  {
    printh(
      "- U základních formátových značek jsou automaticky oddělovačem tzv. bílé\n"
      "  znaky: mezera, tabulátory a konec řádku.\n"
      "- Jakýkoli text vedle formátových značek ve formátovém řetězci je brán\n"
      "  jako oddělovač. => Musí se na vstupu vyskytnout, aby byl vstup bezchybně\n"
      "  přečten.\n"
      "\n"
      "- Mezera ve formátovém řetězci má specifický význam.\n"
      "  => Pokyn \"přeskoč libovolně dlouhou posloupnost bílých znaků\".\n"
      "- Využití to najde při čtení dat ve formátu CSV (typicky ze souborů).\n"
      "- Pokud je první sloupec text s mezerami, je potřeba ve formátovém\n"
      "  řetězci pomocí mezery odstranit znak \\n z předchozího řádku, jinak\n"
      "  se stane součástí načteného řetězce.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka - přeskočení konců řádků")) return;
  if (pokracovat())
  {
    char data[] =
      "  Praha, Vaclavske namesti, 10\n"
      "  Brno, Zelny trh, 1\n";

    char mesto[20+1];
    char ulice[20+1];
    int     cp;
    char mesto1[20+1];
    char ulice1[20+1];
    int     cp1;
    char mesto2[20+1];
    char ulice2[20+1];
    int     cp2;

    // Funkce sscanf čte z textového řetězce data, místo ze standardního vstupu nebo souboru.
    // Tohle je simulace postupného čtení po řádcích ze souboru.
    // V běžném programu by se například ze souboru četlo nějak takto:
    // while (fscanf(vstup, " %20[^,], %20[^,], %d", mesto, ulice, &cp) == 3)
    // {
    //   // Zpracuj data z řádku v proměnných mesto, ulice, cp.
    // }

    sscanf(data, "%20[^,],%20[^,],%d%20[^,],%20[^,],%d", mesto, ulice, &cp, mesto1, ulice1, &cp1);
    sscanf(data, " %20[^,], %20[^,], %d %20[^,], %20[^,], %d ", mesto, ulice, &cp, mesto2, ulice2, &cp2);

    printh(
      "- Mějme na vstupu adresy ve formátu: město, ulice, číslo popisné.\n"
      "\n"
      KOD(
      "  Praha, Vaclavske namesti, 10\n"
      "  Brno, Zelny trh, 1\n"
      )
      "\n"
      PODSEKCE("Co se přečte na druhém řádku")
      "\n"
      "  " KK("\"%%20[^,],%%20[^,],%%d\"    ") " => mesto:|%s|, ulice:|%s|, cp:%d\n"
      "  " KK("\" %%20[^,], %%20[^,], %%d \"") " => mesto:|%s|, ulice:|%s|, cp:%d\n"
      "\n"
      "- V první ukázce se do proměnné " KK("mesto") " načetl i konec řádku a mezery.\n"
      "- Mezera za oddělovačem (čárkou) přeskočí všechny bílé znaky, takže ty\n"
      "  pak nebudou součástí načteného stringu.\n"
      "- Mezera před oddělovačem nemá smysl, protože značka [^,] přečte vše\n"
      "  kromě čárky, tedy i mezery.\n"
      "\n"
      , mesto1, ulice1, cp1
      , mesto2, ulice2, cp2
    );
  }


  if (navratNeboSekce("Zahození slova značkou %%*s")) return;
  if (pokracovat())
  {
    printh(
      "- Použití modifikátoru * při čtení má jiný význam než při tisku.\n"
      "- Značka s modifikátorem * přečte zadaná data, ale nikam je neuloží.\n"
      "  Zahodí je.\n"
      "- K takové značce se ani neočekává žádný odpovídající argument.\n"
      "\n"
      "- Největší využití má značka " KK("%%*s") ".\n"
      "- Tato značka přečte následující slovo a zahodí jej.\n"
      "- To se hodí pro zotavení z chybných vstupů - při detekci chyby se tím\n"
      "  dá přečíst a zahodit chybné slovo a pokračovat v načítání.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka - %%*s")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  printf(\"  Zadej 5 cisel a schvalne dovnitr vloz slovo:\\n\");\n"
      "  float x, suma = 0.0f;\n"
      "  int chyb = 0, i = 0;\n"
      "  while (i < 5)\n"
      "  {\n"
      "    if (scanf(\"%%f\", &x) == 0)\n"
      "    { scanf(\"%%*s\"); chyb += 1; } // Přeskočí chybné slovo.\n"
      "    else\n"
      "    { suma += x; i += 1; }       // Čtení se povedlo.\n"
      "  }\n"
      "  pozerKonecRadku(); // Kdyby po uživateli něco zbylo...\n"
      "  printf(\"  => suma: %g, chyb: %d\\n\");\n"
      )
      "\n"
    );
    char znovu = 'z';
    while (znovu == 'z')
    {
      printf("  Zadej 5 cisel a schvalne dovnitr vloz slovo:\n  ");
      float x, suma = 0.0f;
      int chyb = 0, i = 0, e = 0;
      while (i < 5)
      {
        e = scanf("%f", &x);
        if (e == 0) // Přeskočí chybné slovo.
        {
          e = scanf("%*s");
          chyb += 1;
        }
        else if (e == EOF)
        { break; }
        else
        { suma += x; i += 1; }       // Čtení se povedlo.
      }

      if (e == EOF) // To kdyby někdo ukončil vstup pomocí Ctrl+D v Linuxu nebo Ctrl+Z ve Windows.
        clearerr(stdin);

      pozerKonecRadku(); // Kdyby po uživateli něco zbylo...
      printf(
        "  => suma: %g, chyb: %d\n"
        "\n"
        , suma, chyb
      );

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }


  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void tokenizaceStringu(void)
{
//Rozbití stringu na tokeny
//- sscanf
//- strtok + fgets, gets
//- getchar, fgetc, fgets + realloc, zmínit ungetc

  zacatekKapitoly("Tokenizace stringů");

  printh(
    "- Token je kus textu mezi oddělovači.\n"
    "- Oddělovačem může být jakýkoli text.\n"
    "- Tokenizace je rozbití vstupního textu na tokeny podle zadaných oddělovačů.\n"
    "\n"
    "- Vstupní text je nejčastěji na standardním vstupu nebo v souboru.\n"
    "- Někdy ale dává smysl mít vstupní text, například celý řádek ze vstupního\n"
    "  souboru, načtený ve stringu a tokenizovat jej až následně.\n"
    "\n"
    "- Pro tokenizaci stringů se používají funkce " KK("sscanf") " a " KK("strtok") ".\n"
    "\n"
  );

  if (navratNeboSekce("Přečtení celého řádku ze vstupu")) return;
  if (pokracovat())
  {
    printh(
      "- Funkce " KK("getline") " umí přečíst vstupní řádek libovolné délky.\n"
      "  - Není však v normě ISO C99, takže není vhodná pro přenositelné programy.\n"
      "  => Není jisté, že bude k dispozici na různých operačních systémech.\n"
      "\n"
      "- Funkce " KK("fgets") " umí přečíst vstupní řádek do pole omezené délky.\n"
      "  - To je trochu problém u vstupů, kde se dají čekat velmi dlouhé řádky.\n"
      "  - Dříve existovala i varianta " KK("gets") ". NEPOUŽÍVAT! Není bezpečná.\n"
      "  - Součástí načteného stringu je i znak konce řádku (\\n).\n"
      "\n"
      "- Rozšíření GNU C zapíná ve funkci " KK("scanf") " i značku %%ms, případně %%m[množina],\n"
      "  což umožňuje číst vstup neomezené délky do dynamicky alokovaného pole.\n"
      "  - Uživatel pak nesmí zapomenout použít funkci " KK("free") ".\n"
      "  - Není to součástí normy ISO C99.\n"
      "\n"
      "- Imitaci funkce " KK("getline") " si lze vyrobit svépomocí s využitím funkcí\n"
      "  " KK("fgetc") " a " KK("realloc") ".\n"
      "- Je to nejobecnější řešení, které jde přizpůsobit na míru řešenému problému.\n"
      "- Poznámka: Je-li potřeba vrátit zpět znak (např. při detekci oddělovače)\n"
      "  používá se na to funkce " KK("ungentc") ".\n"
      "\n"
      "=> Obecně bývá lepší dělat tokenizaci přímo nad vstupem než nad stringem.\n"
      "   Funkce " KK("scanf") " je velice mocná a činí dodatečné zpracování celého\n"
      "   přečteného řádku zbytečně pracným řešením. Více viz zdrojový text.\n"
      // Načtení řádku a následné zpracování je časté ve skriptovacích jazycích
      // jako je Javascript, PHP nebo Python. Tyto jazyky nejsou navrženy pro
      // efektivní zpracovávání obrovských objemů dat, takže jim nevadí neefektivita
      // takového řešení.
      // Poznámka: Je to ale ospravedlnitelné řešení u paralelních programů.
      // Pokud nezáleží na pořadí zpracovávaných řádků, může jejich načtení do
      // stringů a následné paralelní zpracování významně urychlit zpracování
      // dat.
      "\n"
    );
  }

  if (navratNeboSekce("Použití sscanf")) return;
  if (pokracovat())
  {
    printh(
      "+ Umí číst ze zadaného stringu stejně, jako čte " KK("scanf") " ze vstupu.\n"
      "+ Umí číst do všech základních datových typů.\n"
      "+ Poradí si s libovolnými oddělovači.\n"
      "+ Vstupní string může být i konstantní string - funkce jej nemění.\n"
      "+ Funkce je relativně bezpečná i v paralelních programech.\n"
      "\n"
      "- Musí z textového řetězce přečíst vše v jednom kroku.\n"
      "  => Nejde ji použít pro postupné zpracování textu v cyklu.\n"
      "  => Počet a pořadí oddělovačů v textu musí být známo předem.\n"
      // Nemá to žádný ekvivalent čtecí hlavy, jako při zpracování souborů.
      // Po zpracování části stringu si nepamatuje, kde skončila, aby v další
      // iteraci cyklu mohla navázat.
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka použití sscanf")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "printf(\"  Zadej text obsahujici 3 carky (max 100 znaku):\\n  \");\n"
      "char radek[100+1];\n"
      "fgets(radek, 100+1, stdin);\n"
      "printf(\"  => radek: |%%s|\\n\", radek);\n"
      "char token[4][100+1];\n"
      "int kod = sscanf(radek, \" %%[^,], %%[^,], %%[^,], %%[^,] \",\n"
      "                 token[0], token[1], token[2], token[3])\n"
      "pozerKonecRadku(); // Kdyby po uživateli něco zbylo...\n"
      )
      "\n"
    );
    char znovu = 'z';
    while (znovu == 'z')
    {
      printf("  Zadej text obsahujici 3 carky (max 100 znaku).\n  ");
      printf("  Napriklad: Jedna, dve, Hozna jde, nese pytel mouky.\n  ");
      char radek[100+1];
      if (fgets(radek, 100+1, stdin) != NULL)
      {
        printf("  => radek: |%s|\n", radek);
        char token[4][100+1];
        int kod = sscanf(radek, " %[^,], %[^,], %[^,], %[^,] ",
                         token[0], token[1], token[2], token[3]);
        //pozerKonecRadku(); // Kdyby po uživateli něco zbylo...
        printf("  => kod: %d\n", kod);
        for (int i = 0; i < kod; ++i)
        {
          printf("  => token[%d]: |%s|\n", i, token[i]);
        }
        printf("  => radek: |%s|\n", radek);
      }
      else
      { // Ošetření, kdyby někdo uzavřel vstup pomocí Ctrl+D v Linuxu nebo
        // Ctrl+Z ve Windows.
        printg("Při čtení došlo k chybě. Asi byl uzavřen stdin.\n");
        clearerr(stdin); // tento hack smaže příznak uzavření
      }

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  if (navratNeboSekce("Použití strtok")) return;
  if (pokracovat())
  {
    printh(
      "+ Rozbije zadaný řetězec na podřetězce - tokeny - podle zadané množiny\n"
      "  oddělovačů.\n"
      "+ Používá se v cyklu.\n"
      "  + V prvním kroku se předá výchozí text.\n"
      "  + V dalších krocích se místo parametru zadává " KK("NULL") ".\n"
      "  + Funkce v dalších iteracích vrací ukazatele na nulou ukončené\n"
      "    stringy, dokud nevrátí " KK("NULL") ".\n"
      "  + String s oddělovači se může v dalších iteracích měnit.\n"
      "  => Počet tokenů v textu nemusí být znám předem.\n"
      "\n"
      "- Umí jen jednoznakové oddělovače - string s oddělovači má význam množiny\n"
      "  znaků, které jsou považovány za oddělovač.\n"
      "- Modifikuje vstupní string.\n"
      "  - Každý nalezený oddělovač přepíše ukončovací nulou.\n"
      "  => Konstantní stringy takto tokenizovat nelze.\n"
      "  => Neumí převést string na číselnou hodnotu.\n"
      "- Zatímco je rozpracován jeden string, nelze funkcí " KK("strtok") " začít\n"
      "  zpracovávat jiný!\n"
//      "  => Pro tyto případy (a např. paralelní zpracování) existuje " KK("strtok_r") ".\n"
//      "     Tato verze ale není ve standardu ISO C99.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka použití strtok")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  printf(\"  Zadej texty oddelovane carkami nebo teckami (max 100 znaku):\\n  \");\n"
      "  char radek[100+1];\n"
      "  fgets(radek, 100+1, stdin);\n"
      "  printf(\"  => radek: |%%s|\\n\", radek);\n"
      "  char *token = strtok(radek, \",.\"); // první volání\n"
      "  while (token != NULL)\n"
      "  {\n"
      "    printf(\"  => token: %%s\\n\", token);\n"
      "    token = strtok(NULL, \",.\"); // následná volání - argumentem je NULL\n"
      "  }\n"
      "  printf(\"  => radek: |%%s|\\n\", radek);\n"
      )
      "\n"
    );

    char znovu = 'z';
    while (znovu == 'z')
    {
      printf("  Zadej texty oddelovane carkami nebo teckami (max 100 znaku):\n  ");
      printf("  Napriklad: Skakal pes, pres oves, pres zelenou louku. Potkal ho myslivec.\n  ");
      char radek[100+1];
      if (fgets(radek, 100+1, stdin) != NULL)
      {
        printf("  => radek: |%s|\n", radek);
        char *token = strtok(radek, ",."); // první volání
        while (token != NULL)
        {
          printf("  => token: |%s|\n", token);
          token = strtok(NULL, ",."); // následná volání - argumentem je NULL
        }
        printf("  => radek: |%s|\n", radek);
      }
      else
      { // Ošetření, kdyby někdo uzavřel vstup pomocí Ctrl+D v Linuxu nebo
        // Ctrl+Z ve Windows.
        printg("Při čtení došlo k chybě. Asi byl uzavřen stdin.\n");
        clearerr(stdin); // tento hack smaže příznak uzavření
      }

      //pozerKonecRadku(); // Kdyby po uživateli něco zbylo...
      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }

    printh(
      "\n"
      "- Na konci je v proměnné " KK("radek") " uložen zdánlivě jen první token.\n"
      "- Ve skutečnosti jsou v poli " KK("radek") " všechny tokeny, ale oddělují je\n"
      "  nulové znaky, kterými byl nahrazen každý nalezený oddělovač.\n"
      "- Jednotlivé vrácené tokeny jsou ukazatele dovnitř původního pole.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void knihovnaString(void)
{
//Operace se stringy
//- strcmp, strcpy, strlen, strcat, strchr, strrchr, strspn, strcspn
//+ varianty strncmp, strncpy, ...

  zacatekKapitoly("Knihovna string.h");

  printh(
    "- Je to obecná knihovna pro práci se stringy.\n"
    "\n"
    "- " KK("strlen ") " - Spočítá délku stringu.\n"
    "\n"
    "- " KK("strcmp ") " - Porovná dva stringy.\n"
    "- " KK("strncmp") " - Porovná dva stringy do zadané délky.\n"
    "- " KK("strcoll") " - Porovná dva stringy podle zapnuté lokalizace.\n"
    "\n"
    "- " KK("strcpy ") " - Kopíruje string do připraveného pole.\n"
    "- " KK("strncpy") " - Kopíruje string do připraveného pole - bezpečnější varianta.\n"
    "- " KK("strcat ") " - Připojí druhý string za první.\n"
    "- " KK("strncat") " - Připojí druhý string za první - bezpečnější varianta.\n"
    "\n"
    "- " KK("strchr ") " - Najde první výskyt znaku ve stringu - hledá zepředu.\n"
    "- " KK("strrchr") " - Najde poslední výskyt znaku ve stringu - hledá odzadu.\n"
    "- " KK("strpbrk") " - Najde první výskyt znaku z množiny ve stringu - hledá zepředu.\n"
    "- " KK("strstr ") " - Najde první výskyt fráze ve stringu.\n"
    "- " KK("strspn ") " - Vrací délku prefixu tvořeného znaky zadané množiny.\n"
    "- " KK("strcspn") " - Vrací délku prefixu tvořeného znaky mimo zadanou množinu.\n"
    "\n"
  );

  if (navratNeboSekce("Délka stringu")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  size_t strlen(const char *s); \n"
      )
      "\n"
      "- Spočítá délku stringu " KK("s") " a vrací ji jako funkční hodnotu.\n"
      "- Pozor! Počítá délku pokaždé znovu.\n"
      "  => Použití v podmínce cyklu je nevhodné!\n"
      "  => V podmínce cyklu je lepší testovat koncovou nulu než délku stringu.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka použití strlen")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  printf(\"  Zadej radek textu (max 100 znaku):\\n  \");\n"
      "  char radek[100+1];\n"
      "  fgets(radek, 100+1, stdin);\n"
      "  printf(\"  => radek: |%%s|\\n\", radek);\n"
      "  printf(\"  => strlen(radek): %%d\\n\" , strlen(radek));\n"
      )
      "\n"
    );

    char znovu = 'z';
    while (znovu == 'z')
    {
      printf("  Zadej radek textu (max 100 znaku):\n  ");
      char radek[100+1];
      if (fgets(radek, 100+1, stdin) != NULL)
      {
        printf("  => radek: |%s|\n", radek);
        printf("  => strlen(radek): %zd\n" , strlen(radek));
      }
      else
      { // Ošetření, kdyby někdo uzavřel vstup pomocí Ctrl+D v Linuxu nebo
        // Ctrl+Z ve Windows.
        printg("Při čtení došlo k chybě. Asi byl uzavřen stdin.\n");
        clearerr(stdin); // tento hack smaže příznak uzavření
      }

      //pozerKonecRadku(); // Kdyby po uživateli něco zbylo...
      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
    printf("\n");
  }


  if (navratNeboSekce("Porovnávání stringů")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "int strcmp(const char *s1, const char *s2);\n"
      )
      "- Porovná dva stringy podle anglické abecedy.\n"
      "\n"
      KOD(
      "int strncmp(const char *s1, const char *s2, size_t n);\n"
      )
      "- Porovná dva stringy do zadané délky podle anglické abecedy.\n"
      "\n"
      KOD(
      "int strcoll(const char *s1, const char *s2);\n"
      )
      "- Porovná dva stringy podle zapnuté lokalizace.\n"
      "- Lokalizaci je potřeba zapnout pomocí funkce " KK("setlocale") ".\n"
      "- Viz dále.\n"
      "\n"
      "- Všechny tři funkce vrací\n"
      "  - nulu, když jsou stringy shodné,\n"
      "  - zápornou hodnotu, když je první string \"menší\", resp. lexikograficky\n"
      "    před druhým stringem,\n"
      "  - kladnou hodnotu, když je první string \"větší\", resp. lexikograficky\n"
      "    za druhým stringem.\n"
      "- V praxi stačí porovnávat s nulou.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka porovnávání stringů")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  if (strcmp(\"atlas\", \"bagr\") >= 0)\n"
      "    printf(\"  => atlas >= bagr - to je špatně\\n\");\n"
      "  else\n"
      "    printf(\"  => atlas < bagr - to je správně\\n\");\n"
      "\n"
      )
    );

    if (strcmp("atlas", "bagr") >= 0)
      printf("  => atlas >= bagr - to je špatně\n");
    else
      printf("  => atlas < bagr - to je správně\n");

    printh(
      KOD(
      "  if (strncmp(\"alternativa\", \"alternator\", 5) == 0)\n"
      "    printf(\"  => alter[nativa] == alter[nator] - to je správně\\n\");\n"
      "  else\n"
      "    printf(\"  => alter[nativa] != alter[nator] - to je špatně\\n\");\n"
      )
      "\n"
    );

    if (strncmp("alternativa", "alternator", 5) == 0)
      printf("  => alter[nativa] == alter[nator] - to je správně\n");
    else
      printf("  => alter[nativa] != alter[nator] - to je špatně\n");

//    printh(
//      "\n"
//      "  setlocale(LC_COLLATE, \"cs_CZ.UTF-8\"); // v Linuxu\n"
//      "  if (strcoll(\"dát\", \"ďas\") < 0)\n"
//      "    printg(\"  => dát < ďas - to je špatně\\n\");\n"
//      "  else\n"
//      "    printg(\"  => dát >= ďas - to je správně\\n\");\n"
//    );
//    setlocale(LC_COLLATE, "cs_CZ.UTF-8"); // v Linuxu
//    if (strcoll("dát", "ďas") < 0)
//      printg("  => dát < ďas - to je špatně\n");
//    else
//      printg("  => dát >= ďas - to je správně\n");
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void ceskaLokalizace(void)
{
//Česká lokalizace
//- strcoll
//  setlocale(LC_ALL, "cs_CZ.UTF-8"); // Linux
//  setlocale(LC_ALL, "Czech_Czechia.852");       // Windows
//  setlocale(LC_ALL, "Czech_Czechia.1250");       // Windows
//- Potíže s českými Windows - UTF-8 vs. ISO-8859-2 vs. CP1250 vs. CP852
//

  zacatekKapitoly("Česká lokalizace");

  printh(
    "- Jazyková lokalizace znamená převod programů do národních jazyků a jejich\n"
    "  kódování.\n"
    "- Pro kódování češtiny na počítačích slouží norma ISO-8859-2, která je\n"
    "  založená na rozšířeném jednobajtovém kódování ASCII.\n"
    "- Dnes se jednobajtová kódování postupně opouštějí ve prospěch vícebajtových\n"
    "  kódování, která dokáží obsáhnout všechny jazyky a jejich abecedy.\n"
    "- Nejrozšířenější je kódování Unicode UTF-8, které je kompatibilní s ASCII,\n"
    "  protože přejímá prvních 127 (jednobajtových) kódů ASCII. Další znaky\n"
    "  (včetně českých) už jsou kódovány ve 2 až 4 bajtech.\n"
    "\n"
    "- Čeština v jazyce C naráží na problémy kódování češtiny v jednotlivých\n"
    "  operačních systémech, kdy zejména Windows nedodržují normu ISO.\n"
    "- Rozdílné standardy používané v Linuxu/unixu a ve Windows jsou hlavním\n"
    "  problémem při tvorbě přenositelných lokalizovaných programů.\n"
    "\n"
    "- Jazyk C byl původně navržen pro ASCII, ale novější normy zavedly i práci\n"
    "  s Unicode. Je však potřeba zapnout lokalizaci a používat upravené\n"
    "  systémové typy a funkce pro práci s tzv. širokými znaky a stringy.\n"
    "\n"
  );

#ifdef __unix__
  char * locale = setlocale(LC_ALL, "cs_CZ.UTF-8");   // Funguje v Linuxu
#else
  char * locale = setlocale(LC_ALL, "Czech_Czechia.852"); // Windows
  //char * locale = setlocale(LC_ALL, "Czech_Czechia.1250"); // Windows
#endif // __unix__

  if (navratNeboSekce("Lokalizace v Linuxu a Windows")) return;
  if (pokracovat())
  {
    printh(
    "- Linuxové systémy mají stejnou lokalizaci jak v GUI, tak v textové konzoli.\n"
    "- Starší verze Linuxů používaly pro češtinu jednobajtový standard ISO-8889-2.\n"
    "- Dnes standard ISO nahradilo vícebajtové kódování UTF-8.\n"
    "- Není však problém přepnout do jiné lokalizace i kódování.\n"
    "\n"
    "- Firma MS vždy oficiální normu v české verzi Windows ignorovala.\n"
    "- V GUI používá vlastní jednobajtové kódování CP1250, které je normě\n"
    "  ISO podobné, ale liší se kódování některých písmen (š, ž).\n"
    "- Horší je, že v textové konzoli používá kódování jiné - CP852.\n"
//    "    zřejmě pro zachování kompatibility s programy pro MS DOS z počátku\n"
//    "    90. let...\n"
    "- Novější programy s GUI využívají i UTF-8.\n"
    "=> Ve Windows se lze setkat se 3 různými způsoby kódování češtiny...\n"
    // S češtinou ve Windows je to tedy jeden velký bordel...
    // Konzoli (cmd.exe) lze spustit i s jiným kódováním, než CP852 - např. cmd.exe /u.
      "\n"
    );
  }

  if (navratNeboSekce("Kdy je potřeba česká lokalizace")) return;
  if (pokracovat())
  {
    printh(
      "- Pro tisk českých znaků není lokalizaci potřeba zapínat.\n"
      "- Stačí tisknout v kódování, jaké používá terminál.\n"
      "- V Linuxu je to dnes obvykle UTF-8 a tisk v něm zvládá i " KK("printf") ".\n"
      "- Ve Windows je to CP852.\n"
      "- Pro tisk UTF-8 je potřeba přepnout terminál do UTF-8.\n"
//      "  - V nastavení Windows nebo parametrem programu (" KK("cmd.exe /u") ").\n"
      "\n"
      "- Pro správné čtení a zpracování lokalizovaných stringů\n"
      "  - v 1bajtovém kódování je potřeba zapnout českou lokalizaci,\n"
      "  - v UTF-8 je potřeba zapnout lokalizaci a pracovat se širokými stringy.\n"
      "- Lokalizaci je také potřeba zapnout pro lokalizovaný tisk čísel\n"
      "  s desetinnou čárkou místo tečky a tisk časových údajů.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Zapnutí české lokalizace funkcí setlocale")) return;
  if (pokracovat())
  {
    printh(
      "\n"
#ifdef __unix__
      KOD(
      "  char * locale = setlocale(LC_ALL, \"cs_CZ.UTF-8\");  // Linux\n"
      "//  char * locale = setlocale(LC_ALL, \"Czech_Czechia.852\");  // Windows\n"
      "//  char * locale = setlocale(LC_ALL, \"Czech_Czechia.1250\");  // Windows\n"
      )
#else
      KOD(
      "//  char * locale = setlocale(LC_ALL, \"cs_CZ.UTF-8\");  // Linux\n"
      "  char * locale = setlocale(LC_ALL, \"Czech_Czechia.852\");  // Windows\n"
      "//  char * locale = setlocale(LC_ALL, \"Czech_Czechia.1250\");  // Windows\n"
      )
#endif // __unix__
      "\n"
      "  => locale: %s\n"
      "\n"
      , locale
    );
    printh(
      "- Na začátku programu je potřeba použít funkci " KK("setlocale") " z " KK("locale.h") ".\n"
      "- První parametr je konstanta udávající, jaké oblasti programu se mají\n"
      "  lokalizovat, např.\n"
      "\n"
      "  " KK("LC_ALL    ") " - všechno\n"
      "  " KK("LC_COLLATE") " - porovnávání stringů\n"
      "  " KK("LC_NUMERIC") " - formát čísel (např. destinná tečka vs. čárka)\n"
      "  " KK("LC_TIME   ") " - formát výpisu data a času\n"
      "\n"
      "- Druhý parametr vybírá jazykovou oblast a kódování ve formátu\n"
      "\n"
      KOD(
      "  jazyk[_oblast][.kódování][@modifikátor]\n"
      )
      "\n"
      "- Pokud se použije parametr \"C\" nebo \"POSIX\", zapne se výchozí ASCII.\n"
      "- Pro češtinu je správný formát \"cs_CZ\" nebo \"cs_CZ.UTF-8\".\n"
      "- Ve Windows funguje \"Czech_Czechia.852\", případně \"Czech_Czechia.1250\".\n"
      // Pokud si uživatel přepne terminál do UTF-8, mohlo by fungovat
      // taky Czech_Czechia.UTF-8.
      // Starší verze Windows používaly také řetězec Czech_Czech Republic.852
      // včetně té mezery.
      // Firma Microsoft není trvale schopná se přizpůsobovat zavedeným standardům...
      "\n"
      "- Funkce vrací ukazatel na string s nastavenou lokalizací, nebo NULL\n"
      "  v případě neúspěchu.\n"
      "\n"
      "- Zapnutá lokalizace ovlivňuje hlavně tyto standardní funkce:\n"
      "  " KK("printf") ", " KK("scanf") ", " KK("strcoll") ", " KK("isalpha") ", " KK("strftime") ".\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void sirokeStringy(void)
{
//- wide char a wide string
//- wchar_t, wchar_t*
//wchar_t * wstring = L"Řetězec";
//wprintf(L"%ls", wstring);
//
//fgetws, wcslen, ... wchar.h
//
//- musí se zapnout správně lokalizace
//- Zdá se, že jeden program nemůže zároveň tisknout na výstup pomocí printf a wprintf.
//- Vypadá to, že první použití jedné z těchto funkcí přepne terminál do svého
//  módu a pak už nic dalšího nefunguje.
//  => Ano, je to tak. Mód terminálu je buďto byte nebo wide - jde zjistit funkcí
//  fwide.
//  => Když se jako první zavolá printf, nastaví se terminál do módu byte,
//     když se jako první zavolá wprintf, nastaví se terminál do módu wide.

  zacatekKapitoly("Široké znaky a stringy");

  printh(
    "- Pro práci s Unicode znaky a stringy zavádí ISO C99 nové typy a funkce.\n"
    "- Tyto nové typy a funkce zavádí modul wchar.h.\n"
    "- Jazyk C pak zavádí novou syntaxi znakových a stringových literálů.\n"
    "\n"
    "- Ke všem běžným funkcím pro stringy existují alternativy pro široké stringy.\n"
    "  např.: " KK("wscanf") ", " KK("wprintf") ", " KK("wcslen") ", " KK("wcscmp") ", ...\n"
    "\n"
    "- Při práci s Unicode je zároveň potřeba zapnout správnou lokalizaci.\n"
    // Důvod je ten, že Unicode obsáhne všechny jazyky a abecedy, ale pro správnou
    // funkčnost je potřeba vědět, v jakém formátu se mají vypisovat desetinná
    // čísla, jak čas a podobně. Tato pravidla zapíná zvolená lokalizace.
    "\n"
  );


  if (navratNeboSekce("Široké typy a konstanty")) return;
  if (pokracovat())
  {
//    wchar_t smajlik = L'☺'; // Znak ve tvaru smajlíku
//    wchar_t text[] = L"Příliš žluťoučký 🐴."; // Obsahuje znak s ikonou koně.

    // V Linuxu to funguje a funkce printf to bez problémů vytiskne na terminál.
    // Pravděpodobně to ale funguje jen v terminálu v GUI. Textový terminál
    // v textovém prostředí to zřejmě nezvládne.

    // Terminál Windows (cmd.exe) takové znaky normálně vytisknout neumí,
    // protože pracuje v CP852. Jde jej přepnout do Unicode spuštěním
    // cmd.exe /u, ale je otázka, zda má takové znaky ve svém fontu.

    printh(
      "\n"
      KOD(
      "  wchar_t smajlik = L'☺'; // Znak ve tvaru smajlíku\n"
      "  wchar_t text[] = L\"Příliš žluťoučký 🐴.\"; // Obsahuje znak s ikonou koně.\n"
      )
      "\n"
      "- Modul wchar.t zavádí typ " KK("wchar_t") ", který slouží pro reprezentaci\n"
      "  širokého, tedy vícebajtového znaku.\n"
      "- Široký string se ukládá do pole tvořeného širokými znaky.\n"
      "\n"
      "- Literál pro široký znak jsou jednoduché uvozovky, před nimiž je znak L.\n"
      "- Literál pro široký string je stejnou analogií obyčejného stringu.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Široký mód datového proudu")) return;
  if (pokracovat())
  {
    printh(
      "- U každého datového proudu nebo souboru jde zjistit a nastavit\n"
      "  široký nebo jednobajtový mód.\n"
      "- Pokud dosud nebyl mód nastaven, první provedená I/O operace tento\n"
      "  mód nastaví.\n"
      "  - Pokud se na standardní výstup nejprve něco vytiskne funkcí " KK("printf") ",\n"
      "    přepne se " KK("stdout") " do jednobajtového módu a funkce pro práci se širokými\n"
      "    znaky a stringy na něm nebudou fungovat.\n"
      "  - Pokud se jako první použije funkce " KK("wprintf") ", přepne to " KK("stdout") "\n"
      "    do širokého módu a běžné funkce jako " KK("printf") " na něm nebudou fungovat.\n"
      "\n"
      "- Explicitně jde mód zapnout funkcí " KK("fwide") ".\n"
      "- Jakmile je jeden mód zapnut, už jej nejde přepnout.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Funkce fwide")) return;
  if (pokracovat())
  {
    printh(
      "- Zjišťuje a zapíná bajtový nebo široký mód pro zpracování zadaného\n"
      "  datového proudu nebo souboru.\n"
      "- Funguje jak se vstupními, tak i výstupními proudy a soubory.\n"
      "\n"
      KOD(
      "  int fwide(FILE *stream, int mode);\n"
      )
      "\n"
      "  " KK("stream") "\n"
      "  - otevřený datový proud nebo soubor\n"
      "\n"
      "  " KK("mode") " \n"
      "  - mód fungování funkce\n"
      "  - 0: Zjišťuje aktuální mód datového proudu.\n"
      "  - kladné číslo: Zkusí přepnout do širokého módu.\n"
      "  - záporné číslo: Zkusí přepnout do jednobajtového módu.\n"
      "\n"
      "- Funkce vrací skutečně nastavený mód.\n"
      "- Pokud vrátí nulu, proud zatím žádný mód nastaven nemá.\n"
      "- Kladné/záporné číslo vrací, když je nastaven široký/bajtový mód.\n"
      "- Vrátí-li jinou hodnotu, než se pokoušíme nastavit, znamená to, že\n"
      "  požadovaný mód zapnout nejde.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Funkce pro široké znaky a stringy")) return;
  if (pokracovat())
  {
    printh(
      "- Ke všem typům a funkcím pro práci ze znaky a stringy existují\n"
      "  široké alternativy.\n"
      "- Modul wchar.h\n"
      "  " KK("EOF   ") " -> " KK("WEOF     ") "\n"
      "  " KK("char  ") " -> " KK("wchar_t  ") "\n"
      "  " KK("char *") " -> " KK("wchar_t *") "\n"
      "\n"
      "  " KK("getchar") " -> " KK("getwchar") "\n"
      "  " KK("putchar") " -> " KK("putwchar") "\n"
      "  " KK("fgetc  ") " -> " KK("fgetwc  ") "\n"
      "  " KK("fputc  ") " -> " KK("fputwc  ") "\n"
      "\n"
      "  " KK("print ") " -> " KK("wprintf ") "\n"
      "  " KK("fprint") " -> " KK("fwprintf") "\n"
      "  " KK("scanf ") " -> " KK("wscanf  ") "\n"
      "  " KK("fscanf") " -> " KK("fwscanf ") "\n"
      "\n"
      "  " KK("strlen ") " -> " KK("wcslen ") "\n"
      "  " KK("strcmp ") " -> " KK("wcscmp ") "\n"
      "  " KK("strcoll") " -> " KK("wcscoll") "\n"
      "  " KK("strcpy ") " -> " KK("wcscpy ") "\n"
      "\n"
      "- Modul wctype.h\n"
      "  " KK("isalpha") " -> " KK("iswalpha") "\n"
      "  " KK("islower") " -> " KK("iswlower") "\n"
      "  " KK("tolower") " -> " KK("towlower") "\n"
      "\n"
      "- Kompletní výčet a dokumentaci modulů wchar.h a wctype.h lze najít\n"
      "  na internetu.\n"
      "\n"
      // V Ubuntu si nainstaluj manuálové stránky z balíků
      // manpages, manpages-dev, manpages-posix a manpages-posix-dev.
    );
  }

  if (navratNeboSekce("Ukázkový široký program")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  char *locale = setlocale(LC_ALL, \"cs_CZ.UTF-8\"); // Linux, možná Mac\n"
      "  int inmode = fwide(stdin, +1);   // +1 znamená široký mód\n"
      "  int outmode = fwide(stdout, +1);\n"
      "\n"
      "  if (outmode < 0) return 1; // Pořád v bajtovém módu\n"
      "\n"
      "  wprintf(\n"
      "    L\"Hurá, jsem v širokém módu!\\n\"\n"
      "    L\"inmode: %%d, outmode: %%d, locale: %%s\\n\"\n"
      "    , inmode, outmode, locale\n"
      "  );\n"
      "\n"
      "  wprintf(L\"Zadej široký text (konec Ctrl+D v Linuxu, Ctrl+Z ve Windows):\\n\");\n"
      "\n"
      "  wchar_t wslovo[20+1];\n"
      "  while (wscanf(L\"%%20ls\", wslovo) == 1)\n"
      "  { wprintf(L\"wslovo: |%%ls|\\n\", wslovo); }\n"
      )
      "\n"
      "- Celý kód najdeš ve zdrojovém souboru.\n"
      "- Musíš si vytvořit nový projekt.\n"
      "- Tady nejde ukázku provést, protože stdin i stdout už jsou přepnuty\n"
      "  do bajtového módu.\n"
      "\n"
    );
/* Ukázkový program pro vyzkoušení. Vyrob si z něj nový projekt.

#include <stdio.h>
#include <stdlib.h>
#include <locale.h>
#include <wchar.h>

int main(void)
{
  // Lokalizace do češtiny
  char *locale = setlocale(LC_ALL, "cs_CZ.UTF-8"); // Linux, možná Mac
//  char *locale = setlocale(LC_ALL, "Czech_Czechia.UTF-8"); // Windows s terminálem přepnutým do UTF-8

  int inmode = fwide(stdin, +1);   // +1 znamená široký mód
  int outmode = fwide(stdout, +1);

  if (outmode < 0)
  {
    printf("Pořád jsem v bajtovém módu. Končím\n");
    return 1;
  }

  wprintf(
    L"Hurá, jsem v širokém módu!\n"
    L"inmode: %d, outmode: %d\n"
    L"locale: %s\n"
    , inmode, outmode
    , locale
  );

  wprintf(
    L"Zadej široký text (konec Ctrl+D v Linuxu, Ctrl+Z ve Windows).\n"
     "Např.: Příliš žluťoučký kůň (🐴) úpěl ďábelské ódy.\n"
  );

  wchar_t wslovo[20+1];
  while (wscanf(L"%20ls", wslovo) == 1)
  { wprintf(L"wslovo: |%ls|\n", wslovo); }
  // Poznámka: %s očekává char *, %ls očekává wchar_t *.
  //           %c očekává char, %lc očekává wchar_t.
  return 0;
}
*/
  }

  konecKapitoly(__func__);
}



